From: Lance Yang <ioworker0@gmail.com>
To: 21cnbao@gmail.com
Cc: akpm@linux-foundation.org, david@redhat.com, ioworker0@gmail.com,
linux-kernel@vger.kernel.org, linux-mm@kvack.org,
mhocko@suse.com, minchan@kernel.org, peterx@redhat.com,
ryan.roberts@arm.com, shy828301@gmail.com,
songmuchun@bytedance.com, wangkefeng.wang@huawei.com,
zokeefe@google.com, fengwei.yin@intel.com
Subject: Re: [PATCH 1/1] mm/madvise: enhance lazyfreeing with mTHP in madvise_free
Date: Mon, 26 Feb 2024 16:37:14 +0800 [thread overview]
Message-ID: <20240226083714.26187-1-ioworker0@gmail.com> (raw)
In-Reply-To: <CAGsJ_4zera4+bWuXHKjdU3QdiR3sqcUQB3gF08DPD49OCT4S9w@mail.gmail.com>
Hey Barry,
Thanks for taking time to review!
On Mon, Feb 26, 2024 at 12:00 PM Barry Song <21cnbao@gmail.com> wrote:
[...]
> On Mon, Feb 26, 2024 at 1:33 AM Lance Yang <ioworker0@gmail.com> wrote:
[...]
> We did something similar on MADV_PAGEOUT[1]
> [1] https://lore.kernel.org/linux-mm/20240118111036.72641-7-21cnbao@gmail.com/
Thanks for providing the link above.
[...]
> > + * Avoid unnecessary folio splitting if the large
> > + * folio is entirely within the given range.
> > + */
> > + folio_test_clear_dirty(folio);
> > + folio_unlock(folio);
> > + for (; addr != next_addr; pte++, addr += PAGE_SIZE) {
> > + ptent = ptep_get(pte);
> > + if (pte_young(ptent) || pte_dirty(ptent)) {
> > + ptent = ptep_get_and_clear_full(
> > + mm, addr, pte, tlb->fullmm);
> > + ptent = pte_mkold(ptent);
> > + ptent = pte_mkclean(ptent);
> > + set_pte_at(mm, addr, pte, ptent);
> > + tlb_remove_tlb_entry(tlb, pte, addr);
> > + }
>
> The code works under the assumption the large folio is entirely mapped
> in all PTEs in the range. This is not always true.
>
> This won't work in some cases as some PTEs might be mapping to the
> large folios. some others might have been unmapped or mapped
> to different folios.
>
> so in MADV_PAGEOUT, we have a function to check the folio is
> really entirely mapped:
>
> +static inline bool pte_range_cont_mapped(unsigned long start_pfn,
> + pte_t *start_pte, unsigned long start_addr, int nr)
> +{
> + int i;
> + pte_t pte_val;
> +
> + for (i = 0; i < nr; i++) {
> + pte_val = ptep_get(start_pte + i);
> +
> + if (pte_none(pte_val))
> + return false;
> +
> + if (pte_pfn(pte_val) != (start_pfn + i))
> + return false;
> + }
> +
> + return true;
> +}
Thanks for providing the information; it's very helpful to me!
I made some changes. Would you mind taking another look, please?
As a diff against this patch.
diff --git a/mm/madvise.c b/mm/madvise.c
index bcbf56595a2e..255d2f329be4 100644
--- a/mm/madvise.c
+++ b/mm/madvise.c
@@ -616,6 +616,18 @@ static long madvise_pageout(struct vm_area_struct *vma,
return 0;
}
+static inline bool pte_range_cont_mapped(pte_t *pte, unsigned long nr)
+{
+ pte_t pte_val;
+ unsigned long pfn = pte_pfn(pte);
+ for (int i = 0; i < nr; i++) {
+ pte_val = ptep_get(pte + i);
+ if (pte_none(pte_val) || pte_pfn(pte_val) != (pfn + i))
+ return false;
+ }
+ return true;
+}
+
static int madvise_free_pte_range(pmd_t *pmd, unsigned long addr,
unsigned long end, struct mm_walk *walk)
@@ -676,20 +688,25 @@ static int madvise_free_pte_range(pmd_t *pmd, unsigned long addr,
*/
if (folio_test_large(folio)) {
int err;
- unsigned long next_addr, align;
+ unsigned long nr, next_addr, align;
if (folio_estimated_sharers(folio) != 1 ||
!folio_trylock(folio))
goto skip_large_folio;
- align = folio_nr_pages(folio) * PAGE_SIZE;
+ nr = folio_nr_pages(folio);
+ align = nr * PAGE_SIZE;
next_addr = ALIGN_DOWN(addr + align, align);
/*
* If we mark only the subpages as lazyfree,
* split the large folio.
*/
- if (next_addr > end || next_addr - addr != align)
+ if (next_addr > end || next_addr - addr != align ||
+ folio_total_mapcount(folio) != nr ||
+ pte_range_cont_mapped(pte, nr))
goto split_large_folio;
/*
---
Thanks again for your time!
Best,
Lance
next prev parent reply other threads:[~2024-02-26 8:37 UTC|newest]
Thread overview: 31+ messages / expand[flat|nested] mbox.gz Atom feed top
2024-02-25 12:32 [PATCH 1/1] mm/madvise: enhance lazyfreeing with mTHP in madvise_free Lance Yang
2024-02-26 2:38 ` Yin Fengwei
2024-02-26 8:35 ` Lance Yang
2024-02-26 12:57 ` Ryan Roberts
2024-02-26 13:03 ` David Hildenbrand
2024-02-26 13:47 ` Lance Yang
2024-02-26 4:00 ` Barry Song
2024-02-26 8:37 ` Lance Yang [this message]
2024-02-26 8:41 ` David Hildenbrand
2024-02-26 8:55 ` Lance Yang
2024-02-26 13:04 ` Ryan Roberts
2024-02-26 13:50 ` Lance Yang
2024-02-27 1:21 ` Barry Song
2024-02-27 1:48 ` Lance Yang
2024-02-27 2:12 ` Barry Song
2024-02-27 2:15 ` Lance Yang
2024-02-26 20:49 ` Barry Song
2024-02-27 1:51 ` Yin Fengwei
2024-02-27 2:17 ` Barry Song
2024-02-27 6:14 ` Yin Fengwei
2024-02-27 6:40 ` Barry Song
2024-02-27 6:42 ` Barry Song
2024-02-27 7:02 ` Yin Fengwei
2024-02-27 7:11 ` Barry Song
2024-02-27 7:21 ` Barry Song
2024-02-27 7:42 ` Yin Fengwei
2024-02-27 7:54 ` Barry Song
2024-02-27 8:33 ` Yin Fengwei
2024-02-27 9:01 ` Barry Song
2024-02-26 13:00 ` Ryan Roberts
2024-02-26 13:54 ` Lance Yang
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20240226083714.26187-1-ioworker0@gmail.com \
--to=ioworker0@gmail.com \
--cc=21cnbao@gmail.com \
--cc=akpm@linux-foundation.org \
--cc=david@redhat.com \
--cc=fengwei.yin@intel.com \
--cc=linux-kernel@vger.kernel.org \
--cc=linux-mm@kvack.org \
--cc=mhocko@suse.com \
--cc=minchan@kernel.org \
--cc=peterx@redhat.com \
--cc=ryan.roberts@arm.com \
--cc=shy828301@gmail.com \
--cc=songmuchun@bytedance.com \
--cc=wangkefeng.wang@huawei.com \
--cc=zokeefe@google.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -676,20 +688,25 @@ static int madvise_free_pte_range(pmd_t *pmd, unsigned long addr,
next_addr = ALIGN_DOWN(addr + align, align);
/*
- * If we mark only the subpages as lazyfree,
- * split the large folio.
+ * If we mark only the subpages as lazyfree, or
+ * if there is a cow folio associated with this folio,
+ * or if this folio is not really entirely mapped,
+ * then split the large folio.
*/
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.